@@ -3,6 +3,27 @@
 typedef struct item Item;
 typedef struct dir Dir;
 
+enum {
+	TXT,
+	DIR,
+	CSO,
+	ERR,
+	MAC,
+	DOS,
+	UUE,
+	IND,
+	TLN,
+	BIN,
+	MIR,
+	IBM,
+	GIF,
+	IMG,
+	URL,
+	INF,
+	UNK,
+	BRK,
+};
+
 struct item {
 	char type;
 	char redtype;
@@ -23,6 +44,7 @@ struct dir {
 	size_t curline;
 };
 
+extern void (*diag)(char *fmt, ...);
 void die(const char *fmt, ...);
 size_t mbsprint(const char *s, size_t len);
 #ifdef NEED_STRCASESTR
