@@ -7,16 +7,25 @@
 #include <unistd.h>
 #include <sys/types.h>
 
-#include "config.h"
 #include "common.h"
+#include "config.h"
 
 #define C(c) #c
 #define S(c) C(c)
 
+/* ncurses doesn't define those in term.h, where they're used */
+#ifndef OK
+#define OK (0)
+#endif
+#ifndef ERR
+#define ERR (-1)
+#endif
+
 static char bufout[256];
 static struct termios tsave;
 static struct termios tsacc;
 static Item *curentry;
+static int termset = ERR;
 
 void
 uisetup(void)
@@ -28,7 +37,9 @@ uisetup(void)
 	tsacc.c_cc[VTIME] = 0;
 	tcsetattr(0, TCSANOW, &tsacc);
 
-	setupterm(NULL, 1, NULL);
+	if (termset != OK)
+		/* setupterm call exits on error */
+		termset = setupterm(NULL, 1, NULL);
 	putp(tparm(clear_screen, 0, 0, 0, 0, 0, 0, 0, 0, 0));
 	putp(tparm(save_cursor, 0, 0, 0, 0, 0, 0, 0, 0, 0));
 	putp(tparm(change_scroll_region, 0, lines-2, 0, 0, 0, 0, 0, 0, 0));
@@ -39,9 +50,13 @@ uisetup(void)
 void
 uicleanup(void)
 {
+	tcsetattr(0, TCSANOW, &tsave);
+
+	if (termset != OK)
+		return;
+
 	putp(tparm(change_scroll_region, 0, lines-1, 0, 0, 0, 0, 0, 0, 0));
 	putp(tparm(clear_screen, 0, 0, 0, 0, 0, 0, 0, 0, 0));
-	tcsetattr(0, TCSANOW, &tsave);
 	fflush(stdout);
 }
 
@@ -82,7 +97,7 @@ uiprompt(char *fmt, ...)
 	putp(tparm(restore_cursor, 0, 0, 0, 0, 0, 0, 0, 0, 0));
 	fflush(stdout);
 
-	if (r < 0) {
+	if (r == -1) {
 		clearerr(stdin);
 		clear(&input);
 	} else if (input[r - 1] == '\n') {
@@ -565,14 +580,16 @@ uisigwinch(int signal)
 {
 	Dir *dir;
 
-	setupterm(NULL, 1, NULL);
+	if (termset == OK)
+		del_curterm(cur_term);
+	termset = setupterm(NULL, 1, NULL);
 	putp(tparm(change_scroll_region, 0, lines-2, 0, 0, 0, 0, 0, 0, 0));
 
 	if (!curentry || !(dir = curentry->dat))
 		return;
 
 	if (dir->curline - dir->printoff > lines-2)
-		dir->curline = dir->printoff + lines-2;
+		dir->printoff = dir->curline - (lines-2);
 
 	uidisplay(curentry);
 }
